<?php
/**
 Plugin Name: Weighted Random Authors
 Plugin URI: http://www.cmurrayconsulting.com/software/wordpress-weighted-random-authors/
 Description: A widget that selects a limited list of <strong>random authors, weighted towards authors with more posts</strong>. Configure numer of authors, title, display of post count, and author gravatar display.  
 Version: 1.0.1
 Author: Jacob M Goldman (C. Murray Consulting)
 Author URI: http://www.cmurrayconsulting.com

    Plugin: Copyright 2009 C. Murray Consulting  (email : jake@cmurrayconsulting.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class WeightedAuthorWidget extends WP_Widget
{
	function WeightedAuthorWidget() {
		$widget_ops = array('classname' => 'widget_weighted_authors', 'description' => __( "Displays authors in random order, weighted by number of posts.") );
		$this->WP_Widget('weightedauthors', __('Weighted Random Authors'), $widget_ops);
	}

    function widget($args, $instance) {
		extract($args);
      
		$title = apply_filters('widget_title', empty($instance['title']) ? 'Random Authors' : $instance['title']);
		$limit = (is_numeric($instance['limit']) && $instance['limit'] >= 0) ? intval($instance['limit']) : 5;
		
		echo $before_widget;		
		echo $before_title.$title.$after_title."<ul>";

      	//the magic
      	global $wpdb;
      	$sqllimit = ($instance['limit'] > 0) ? " LIMIT ".$limit : '';
      	$authors = $wpdb->get_results("SELECT post_author, count(ID) as count FROM $wpdb->posts WHERE post_type = 'post' AND ".get_private_posts_cap_sql('post')." GROUP BY post_author ORDER BY ID*rand() DESC$sqllimit");
      	
      	foreach($authors as $author) {      		
      		echo "<li>";
      		
      		$author_info = get_userdata($author->post_author);
			$name = ($author_info->first_name && $author_info->last_name) ? $author_info->first_name.' '.$author_info->last_name : $author_info->display_name;
			
			if (isset($instance['gravatar']) && $instance['gravatar']) echo get_avatar($author_info->user_email,"16")." ";
      		echo '<a href="'.get_author_posts_url($author_info->ID).'" title="'.esc_attr(sprintf(__("Posts by %s"), $author_info->display_name)).'">'.$name.'</a>';
      		if (isset($instance['show_count']) && $instance['show_count']) echo " (".$author->count.")"; 
      		
      		echo "</li>\n";
      	}
      	      	
		echo "</ul>".$after_widget;
	}

	function update($new_instance, $old_instance){
		$instance = $old_instance;
		$instance['title'] = strip_tags(stripslashes($new_instance['title']));
		$instance['limit'] = intval($new_instance['limit']);
		$instance['show_count'] = $new_instance['show_count'];
		$instance['gravatar'] = $new_instance['gravatar'];
		return $instance;
	}

	function form($instance){
		$instance = wp_parse_args((array) $instance, array('title' => 'Random Authors', 'limit' => 5, 'show_count' => true, 'gravatar' => false)); //defaults
		
		$title = htmlspecialchars($instance['title']);
		$limit = (is_numeric($instance['limit']) && intval($instance['limit']) >= 0) ? intval($instance['limit']) : 5;
		$show_count_checked = ($instance['show_count']) ? 'checked="checked" ' : '';
		$gravatar_checked = ($instance['gravatar']) ? 'checked="checked" ' : '';
				
		echo '<p><label for="'.$this->get_field_name('title').'">'.__('Title:').' <input type="text" id="'.$this->get_field_id('title').'" name="'.$this->get_field_name('title').'" value="'.$title.'" /></label></p>';
		echo '<p><label for="'.$this->get_field_name('limit').'">'.__('Limit:').' <input type="text" id="'.$this->get_field_id('limit').'" name="'.$this->get_field_name('limit').'" value="'.$limit.'" size="3" /> <small>set to 0 for all</small></label></p>';
		echo '<p><label for="'.$this->get_field_name('show_count').'">'.__('Show Count:').' <input type="checkbox" id="'.$this->get_field_id('show_count').'" name="'.$this->get_field_name('show_count').'" '.$show_count_checked.'/></label></p>';
		echo '<p><label for="'.$this->get_field_name('gravatar').'">'.__('Show Avatar:').' <input type="checkbox" id="'.$this->get_field_id('gravatar').'" name="'.$this->get_field_name('gravatar').'" '.$gravatar_checked.'/></label></p>';
	}

}

function WeightedAuthorInit() { register_widget('WeightedAuthorWidget'); }
add_action('widgets_init', 'WeightedAuthorInit');

?>